﻿using System.Reflection;
using C1.WPF.Chart;

namespace ColumnChart
{
    public class CustomFlexChart : C1FlexChart
    {
        public CustomFlexChart()
        {
            Loaded += CustomFlexChart_Loaded;
            Rendered += CustomFlexChart_Rendered;
        }
        
        private void CustomFlexChart_Rendered(object sender, RenderEventArgs e)
        {
            if (sender is C1FlexChart chart)
            {
                var baseChartType = chart.GetType().BaseType;
                if (baseChartType != null)
                {
                    var chartField =
                        baseChartType.GetField("_chartRenderer", BindingFlags.Instance | BindingFlags.NonPublic);
                    if (chartField != null)
                    {
                        var flags = BindingFlags.Instance | BindingFlags.NonPublic;
                        var renderer = chartField.GetValue(chart);
                        var mGetPlotter = renderer.GetType().GetMethod("GetPlotter", flags, null,
                            new[] {typeof(C1.Chart.ISeries)}, null);
                        var mRenderData = renderer.GetType().GetMethod("RenderData", flags);
                        if (mGetPlotter != null)
                        {
                            var pl = mGetPlotter.Invoke(renderer, new object[] {null});
                            renderer.GetType().GetMethod("ClearPlotters", flags)?.Invoke(renderer, null);
                            mRenderData?.Invoke(renderer, new[] {chart, e.Engine, pl});
                        }
                    }
                }

            }
        }

        private void CustomFlexChart_Loaded(object sender, System.Windows.RoutedEventArgs e)
        {
            RenderMode = RenderMode.Direct2D;

            foreach (var seriesObject in Series)
            {
                seriesObject.SymbolRendering += OnSymbolRendered;
            }
            
        }

        private void OnSymbolRendered(object sender, RenderSymbolEventArgs e)
        {
            if (!(sender is Series series)) return;

            // Hide the grid lines from the render area.
            if (series.Style == null)
                e.Engine.SetFill(((C1.Chart.IPalette)this).GetColor(this.Series.IndexOf(series)));
        }
    }
}
